package com.example.ueberwachungssystem.Detector;

import android.content.Context;
import androidx.annotation.NonNull;


abstract public class Detector {
    private OnDetectionListener listener;

    /** Constructor - takes context of current activity */
    public Detector(Context context) {};


    /** On Detection Listener - runs when violation is reported */
    public interface OnDetectionListener {
        void onDetection(@NonNull DetectionReport detectionReport);
    }
    public void setOnDetectionListener(@NonNull OnDetectionListener listener) {
        this.listener = listener;
    }


    /** Triggers onDetectionListener - call this to trigger violation/alarm */
    public void reportViolation(String detectorID, String detectionType, float amplitude) {
        if (listener != null) {
            DetectionReport detectionReport = new DetectionReport(detectorID, detectionType, amplitude);
            listener.onDetection(detectionReport);
        } else {
            throw new IllegalStateException("No listener set for violation reporting");
        }
    }


    /** Starts Detection (abstract method: needs to be overridden in child class) */
    public abstract void startDetection();

    /** Stops Detection (abstract method: needs to be overridden in child class) */
    public abstract void stopDetection();
}